@@ -1,6 +1,6 @@
 # Author: Babak Naimi, naimi.b@gmail.com
-# Date :  Oct. 2016
-# Version 2.5
+# Date (last update):  July 2017
+# Version 2.9
 # Licence GPL v3
 #--------
 
@@ -23,8 +23,11 @@
   n
 }
 #----------
-.getSpeciesDistribution <- function(data) {
-  o <- lapply(data@species,function(x) {
+.getSpeciesDistribution <- function(data,sp=NULL) {
+  if (!is.null(sp)) sp <- data@species[sp]
+  else sp <- data@species
+  
+  o <- lapply(sp,function(x) {
     if (!is.null(x@presence)) return('binomial')
     else if (!is.null(x@abundance)) return('poisson')
     else if (!is.null(x@Multinomial)) return('multinomial')
@@ -305,7 +308,15 @@
     x[[sp]]$test$rID[x[[sp]]$test$rowID %in% id]
   }
 }
-
+#--------------
+# convert a character vector to a list and use the values as the names of the list item unless the names are specified through n
+.charVector2List <- function(x,n=NULL) {
+      xx <- as.list(x)
+      if (!is.null(n) && length(n) == length(x)) names(xx) <- n
+      else names(xx) <- x
+      xx
+}
+#------------
 .generateWL <- function(d,s) {
   pkgs <- .sdmMethods$getPackageNames(s@methods)
   .sdm...temp <- NULL; rm(.sdm...temp)
@@ -348,7 +359,9 @@
       s@methods <- s@methods[ww]
     }
   }
-  
+  #------------
+  if (!is.null(s@seed)) set.seed(s@seed)
+  #-----------
   #fr <- .getFeaturetype(d,s@sdmFormula)
   w <- new('.workload',ncore=s@ncore,data=d,setting=s,frame=s@featuresFrame)
   hasTest <- 'test' %in% d@groups$training@values[,2]
@@ -400,37 +413,123 @@
       }
     }
   }
-  
+  #------------
   w$funs[['fit']] <- .sdmMethods$getFitFunctions(s@methods)
   w$arguments[['fit']] <- .sdmMethods$getFitArguments(s@methods)
   w$funs[['predict']] <- .sdmMethods$getPredictFunctions(s@methods)
   w$arguments[['predict']] <- .sdmMethods$getPredictArguments(s@methods)
+  #---
+  #------ applying the setting rules & overriding the user settings:
+  for (mo in w$setting@methods) {
+    .u <- NULL
+    if (!is.null(w$setting@modelSettings) && mo %in% names(w$setting@modelSettings)) {
+      .u <- w$setting@modelSettings[[mo]]
+    }
+    for (sp in names(w$train)) {
+      w$arguments[['overriden_settings']][['fit']][[mo]][[sp]] <- w$arguments$fit[[mo]]$settings
+      w$arguments[['overriden_settings']][['predict']][[mo]][[sp]] <- w$arguments$predict[[mo]]$settings
+      #-------
+      .set <- w$setRules(mo,sp)
+      if (!is.null(.set)) {
+        if ('fitSettings' %in% names(.set)) {
+          w$arguments[['overriden_settings']][['fit']][[mo]][[sp]][names(.set[['fitSettings']])] <- .set[['fitSettings']]
+        }
+        
+        if ('predictSettings' %in% names(.set)) {
+          w$arguments[['overriden_settings']][['predict']][[mo]][[sp]][names(.set[['predictSettings']])] <- .set[['predictSettings']]
+        }
+      }
+      #-------
+      if (!is.null(.u)) {
+        if (any(names(.u) %in% c('fitSettings','fitSetting','fitsetting','fitset','FitSetting','FitSettings','predictSettings','predictSetting','predictsetting','predictset','PredictSetting','PredictSettings'))) {
+          .w <- which(names(.u) %in% c('fitSettings','fitSetting','fitsetting','fitset','FitSetting','FitSettings'))
+          if (length(.w) == 1) {
+            if (is.list(.u[[.w]])) {
+              .ww <- names(.u[[.w]]) %in% names(w$arguments[['overriden_settings']][['fit']][[mo]][[sp]])
+              if (any(.ww)) {
+                w$arguments[['overriden_settings']][['fit']][[mo]][[sp]][names(.u[[.w]])[.ww]] <- .u[[.w]][.ww]
+                if (any(!.ww)) w$arguments[['overriden_settings']][['fit']][[mo]][[sp]] <- c(w$arguments[['overriden_settings']][['fit']][[mo]][[sp]],.u[[.w]][!.ww])
+              } else w$arguments[['overriden_settings']][['fit']][[mo]][[sp]] <- c(w$arguments[['overriden_settings']][['fit']][[mo]][[sp]],.u[[.w]])
+            }
+            .u <- .u[-.w]
+          }
+          #------
+          .w <- which(names(.u) %in% c('predictSettings','predictSetting','predictsetting','predictset','PredictSetting','PredictSettings'))
+          if (length(.w) == 1) {
+            if (is.list(.u[[.w]])) {
+              .ww <- names(.u[[.w]]) %in% names(w$arguments[['overriden_settings']][['predict']][[mo]][[sp]])
+              if (any(.ww)) {
+                w$arguments[['overriden_settings']][['predict']][[mo]][[sp]][names(.u[[.w]])[.ww]] <- .u[[.w]][.ww]
+                if (any(!.ww)) w$arguments[['overriden_settings']][['predict']][[mo]][[sp]] <- c(w$arguments[['overriden_settings']][['predict']][[mo]][[sp]],.u[[.w]][!.ww])
+              } else w$arguments[['overriden_settings']][['predict']][[mo]][[sp]] <- c(w$arguments[['overriden_settings']][['predict']][[mo]][[sp]],.u[[.w]])
+            }
+            .u <- .u[-.w]
+          }
+        }
+        #------
+        if (length(.u) > 0) {
+          
+          .ww <- names(.u) %in% names(w$arguments[['overriden_settings']][['fit']][[mo]][[sp]])
+          if (any(.ww)) {
+            w$arguments[['overriden_settings']][['fit']][[mo]][[sp]][names(.u)[.ww]] <- .u[.ww]
+            if (any(!.ww)) w$arguments[['overriden_settings']][['fit']][[mo]][[sp]] <- c(w$arguments[['overriden_settings']][['fit']][[mo]][[sp]],.u[!.ww])
+          } else w$arguments[['overriden_settings']][['fit']][[mo]][[sp]] <- c(w$arguments[['overriden_settings']][['fit']][[mo]][[sp]],.u)
+        }
+      }
+    }
+  }
+  
+  #-------------
+  
   #w$dataObject.names <- unique(unlist(lapply(s@methods, .sdmMethods$getDataArgumentNames)))
   mo <- s@methods
   names(mo) <- mo
   w$dataObject.names <- lapply(mo, .sdmMethods$getDataArgumentNames)
+  w$settingRules <- lapply(mo,function(x) .sdmMethods$Methods[[x]]@settingRules)
   #-----------
   
   #reserved.names <- w$getReseved.names()
   for (mo in s@methods) {
     wc <- unlist(lapply(w$arguments$fit[[mo]]$params,function(x) is.character(x)))
     if (any(!wc)) {
-      if (!all(unlist(lapply(w$arguments$fit[[mo]]$params[!wc],function(x) is.function(x))))) stop(paste('parameter definition for the model',mo,'in the model container is not correctly defined!'))
-      for (n in names(w$arguments$fit[[mo]]$params[!wc])) {
-        #if (!all(names(formals(w$arguments$fit[[mo]]$params[[n]])) %in% reserved.names)) stop(paste('the input argument for the function generates the parameter for model',m,'is unknown (not in the reseved objects)'))
-        w$params[[sp]][[n]] <- do.call(w$arguments$fit[[mo]]$params[[n]],w$generateParams(names(formals(w$arguments$fit[[mo]]$params[[n]])),sp)) 
-        w$arguments$fit[[mo]]$params[[n]] <- n
+      if (!all(unlist(lapply(w$arguments$fit[[mo]]$params[!wc],function(x) is.function(x))))) {
+        warning(paste('parameter definition for the model',mo,'in the model container is not correctly defined!'))
+        cat('\nmethod',mo,'is removed because the definition for the fit parameters',paste(names(w$arguments$fit[[mo]]$params)[!wc],collapse=', '),' is not correct!\n')
+        w$setting@methods <- .excludeVector(w$setting@methods,mo)
+        w$arguments$fit <- w$arguments$fit[-which(names(w$arguments$fit) == mo)]
+        w$arguments$predict <- w$arguments$predict[-which(names(w$arguments$predict) == mo)]
+        w$funs$fit <- w$funs$fit[-which(names(w$funs$fit) == mo)]
+        w$funs$predict <- w$funs$predict[-which(names(w$funs$predict) == mo)]
+        w$dataObject.names <- w$dataObject.names[-which(names(w$dataObject.names) == mo)]
+      } else {
+        for (n in names(w$arguments$fit[[mo]]$params)[!wc]) {
+          .nn <- paste0(n,'_',mo,'_fit') # a specific name is generated and added to w$params to which the function is assigned
+          # the assigned function will be called when the w$generateParams is executed to generate the customised parameters defined through the function in the model definition
+          w$params[[.nn]] <- w$arguments$fit[[mo]]$params[[n]]
+          w$arguments$fit[[mo]]$params[[n]] <- .nn
+        }
       }
     }
-    
-    wc <- unlist(lapply(w$arguments$predict[[mo]]$params,function(x) is.character(x)))
-    
-    if (any(!wc)) {
-      if (!all(unlist(lapply(w$arguments$predict[[mo]]$params[!wc],function(x) is.function(x))))) stop(paste('parameter definition for the model',mo,'in the model container is not correctly defined!'))
-      for (n in names(w$arguments$predict[[mo]]$params[!wc])) {
-        #if (!all(names(formals(w$arguments$predict[[mo]]$params[[n]])) %in% reserved.names)) stop(paste('the input argument for the function generates the parameter for model',m,'is unknown (not in the reseved objects)'))
-        w$params[[sp]][[n]] <- do.call(w$arguments$predict[[mo]]$params[[n]],w$generateParams(names(formals(w$arguments$predict[[mo]]$params[[n]])),sp))
-        w$arguments$predict[[mo]]$params[[n]] <- n
+    #-----
+    if (mo %in% w$setting@methods) {
+      wc <- unlist(lapply(w$arguments$predict[[mo]]$params,function(x) is.character(x)))
+      if (any(!wc)) {
+        if (!all(unlist(lapply(w$arguments$predict[[mo]]$params[!wc],function(x) is.function(x))))) {
+          warning(paste('parameter definition for the model',mo,'in the model container is not correctly defined!'))
+          cat('\nmethod',mo,'is removed because the definition for the predict parameters',paste(names(w$arguments$predict[[mo]]$params)[!wc],collapse=', '),' is not correct!\n')
+          w$setting@methods <- .excludeVector(w$setting@methods,mo)
+          w$arguments$fit <- w$arguments$fit[-which(names(w$arguments$fit) == mo)]
+          w$arguments$predict <- w$arguments$predict[-which(names(w$arguments$predict) == mo)]
+          w$funs$fit <- w$funs$fit[-which(names(w$funs$fit) == mo)]
+          w$funs$predict <- w$funs$predict[-which(names(w$funs$predict) == mo)]
+          w$dataObject.names <- w$dataObject.names[-which(names(w$dataObject.names) == mo)]
+        } else {
+          for (n in names(w$arguments$predict[[mo]]$params)[!wc]) {
+            .nn <- paste0(n,'_',mo,'_predict') 
+            w$params[[.nn]] <- w$arguments$predict[[mo]]$params[[n]]
+            w$arguments$predict[[mo]]$params[[n]] <- .nn
+          }
+        }
       }
     }
   }
@@ -475,17 +574,199 @@
 }
 
 #----------------------------------------
+# if (!isGeneric("sdmSetting")) {
+#   setGeneric("sdmSetting", function(formula,data,methods,interaction.depth=1,n=1,replication=NULL,
+#                                     cv.folds=NULL,test.percent=NULL,bg=NULL,bg.n=NULL,var.importance=NULL,response.curve=TRUE,
+#                                     var.selection=FALSE,ncore=1L,...)
+#     standardGeneric("sdmSetting"))
+# }
+# 
+# setMethod('sdmSetting', signature(formula='ANY','sdmdata','character'), 
+#           function(formula,data,methods,interaction.depth=1,n=1,replication=NULL,
+#                    cv.folds=NULL,test.percent=NULL,bg=NULL,bg.n=NULL,var.importance=NULL,response.curve=TRUE,
+#                    var.selection=FALSE,ncore=1L,...) {
+#             
+#             if (!.sdmOptions$getOption('sdmLoaded')) .addMethods()
+#             
+#             dot <- list(...)
+#             sobj <- NULL
+#             if (length(dot) > 0) {
+#               ndot <- names(dot)
+#               if ('' %in% ndot) {
+#                 for (i in seq_along(which(ndot == ''))) {
+#                   if (inherits(dot[[i]],'.sdmCorSetting')) {
+#                     sobj <- dot[[i]]
+#                     break
+#                   }
+#                 }
+#                 dot <- dot[-which(ndot == '')]
+#                 ndot <- names(dot)
+#               }
+#               
+#               a <- c('interaction.depth','replication','cv.folds','test.percent','bg','bg.n','var.importance','response.curve','var.selection','ncore')
+#               ndot <- .pmatch(ndot,a)
+#               w <- !is.na(ndot)
+#               if (length(w) > 0) {
+#                 dot <- dot[w]
+#                 ndot <- ndot[w]
+#                 names(dot) <- ndot
+#               }
+#               
+#               
+#               if ('setting' %in% names(dot) && inherits(dot[['setting']],'.sdmCorSetting')) {
+#                 sobj <- dot[['setting']]
+#                 dot <- dot[-which(ndot == 'setting')]
+#                 ndot <- names(dot)
+#               }
+#               
+#               if (length(dot) > 0) {
+#                 if (length(ndot) > 0) {
+#                   for (nd in ndot) {
+#                     if (nd == 'interaction.depth' && interaction.depth == 1) interaction.depth <- dot[[nd]]
+#                     else if (nd == 'ncore' && ncore == 1L) ncore <- dot[[nd]]
+#                     else if (nd == 'replication' && is.null(replication)) replication <- dot[[nd]]
+#                     else if (nd == 'cv.folds' && is.null(cv.folds)) cv.folds <- dot[[nd]]
+#                     else if (nd == 'test.percent' && is.null(test.percent)) test.percent <- dot[[nd]]
+#                     else if (nd == 'bg' && is.null(bg)) bg <- dot[[nd]]
+#                     else if (nd == 'bg.n' && is.null(bg.n)) bg.n <- dot[[nd]]
+#                     else if (nd == 'var.importance' && is.null(var.importance)) var.importance <- dot[[nd]]
+#                     else if (nd == 'response.curve' && response.curve && is.logical(dot[[nd]])) response.curve <- dot[[nd]]
+#                     else if (nd == 'var.selection' && !var.selection && is.logical(dot[[nd]])) var.selection <- dot[[nd]]
+#                   }
+#                 }
+#               }
+#             }
+#             #--------
+#             
+#             m <- .methodFix(methods)
+#             if (any(is.na(m))) stop(paste('methods',paste(methods[is.na(m)],collapse=', '),'do not exist!'))
+#             m <- unique(m)
+#             
+#             s <- new('.sdmCorSetting',methods=m)
+#             s@distribution <- .getSpeciesDistribution(data)
+#             
+#             if (missing(formula)) {
+#               if (!is.null(sobj)) {
+#                 if (all(sobj@sdmFormula@vars %in% data@features.name)) s@sdmFormula <- sobj@sdmFormula
+#                 else s@sdmFormula <- data@sdmFormula
+#               } else s@sdmFormula <- data@sdmFormula
+#               
+#             } else if (inherits(formula,'sdmFormula')) s@sdmFormula <- formula
+#             else if (inherits(formula,'formula')) {  
+#               s@sdmFormula <- .exFormula(formula,as.data.frame(data)[,-1])
+#             } else if (inherits(formula,'.sdmCorSetting')) {
+#               sobj <- formula
+#               if (all(sobj@sdmFormula@vars %in% data@features.name)) s@sdmFormula <- sobj@sdmFormula
+#               else s@sdmFormula <- data@sdmFormula
+#             } else {
+#               if (!is.null(sobj)) {
+#                 if (all(sobj@sdmFormula@vars %in% data@features.name)) s@sdmFormula <- sobj@sdmFormula
+#                 else s@sdmFormula <- data@sdmFormula
+#               } else s@sdmFormula <- data@sdmFormula
+#             }
+#             
+#             s@featuresFrame <- .getFeaturetype(data,s@sdmFormula)  
+#               
+#             if (!is.null(test.percent)) s@test.percentage <- test.percent
+#             else {
+#               if (!is.null(sobj)) {
+#                 if (!is.null(sobj@test.percent)) s@test.percentage <- sobj@test.percent
+#               }
+#             }
+#             
+#             s@interaction.depth <- interaction.depth
+#             if (interaction.depth ==1 && !is.null(sobj) && !is.null(sobj@interaction.depth)) s@interaction.depth <- sobj@interaction.depth
+#             
+#             s@ncore <- ncore
+#             if (ncore == 1L && !is.null(sobj) && length(sobj@ncore) == 1) s@ncore <- sobj@ncore
+#             
+#             if (!is.null(replication)) {
+#               nx <- .replicate.methodFix(replication)
+#               if (any(is.na(nx))) warning(paste(paste(replication[is.na(nx)],collapse=', '),'methods in replication are not found [They are ignored!]'))
+#               replication <- nx[!is.na(nx)]
+#               s@replicate <- replication
+#             } else {
+#               if (!is.null(sobj)) {
+#                 if (!is.null(sobj@replicate)) s@replicate <- sobj@replicate
+#               }
+#               if (is.null(s@replicate) && !is.null(s@test.percentage)) {
+#                 s@replicate <- "subsampling"
+#               }
+#             }
+#             
+#             s@n.replicates <- n
+#             if (!is.null(sobj) && !is.null(sobj@n.replicates)) s@n.replicates <- sobj@n.replicates
+#             
+#             if ("subsampling" %in% s@replicate) {
+#               if (is.null(s@test.percentage)) s@test.percentage <- 30
+#             }
+#             
+#             if (!is.null(cv.folds)) s@cv.folds <- cv.folds
+#             else {
+#               if (!is.null(sobj) && !is.null(sobj@cv.folds)) s@cv.folds <- sobj@cv.folds
+#               if (is.null(s@cv.folds) && "cross_validation" %in% s@replicate) s@cv.folds <- 5
+#             }
+#             
+#             if (!is.null(s@cv.folds) && !"cross_validation" %in% s@replicate) {
+#               s@replicate <- c("cross_validation",s@replicate)
+#             }
+#             
+#             if (!is.null(bg)) s@pseudo.absence.methods <- bg
+#             else {
+#               if (!is.null(sobj)) {
+#                 if (!is.null(sobj@pseudo.absence.methods)) s@pseudo.absence.methods <- sobj@pseudo.absence.methods
+#               }
+#             }
+#             if (!is.null(bg.n)) s@n.pseudo.absence <- bg.n
+#             else {
+#               if (!is.null(sobj)) {
+#                 if (!is.null(sobj@n.pseudo.absence)) s@n.pseudo.absence <- sobj@n.pseudo.absence
+#               }
+#               if (is.null(s@n.pseudo.absence) && !is.null(s@pseudo.absence.methods)) {
+#                 s@n.pseudo.absence <- 1000
+#               }
+#             }
+#             if (!is.null(var.importance)) s@varImportance.methods <- var.importance
+#             else {
+#               if (!is.null(sobj)) {
+#                 if (!is.null(sobj@varImportance.methods)) s@varImportance.methods <- sobj@varImportance.methods
+#               }
+#             }
+#             if (response.curve) s@response.curve <- TRUE
+#             else {
+#               if (!is.null(sobj)) {
+#                 if (!is.null(sobj@response.curve) && sobj@response.curve) s@response.curve <- sobj@response.curve
+#               } else s@response.curve <- FALSE
+#             }
+#             
+#             if (var.selection) s@var.selection <- TRUE
+#             else {
+#               if (!is.null(sobj)) {
+#                 if (!is.null(sobj@var.selection) && sobj@var.selection) s@var.selection <- sobj@var.selection
+#               } else s@var.selection <- FALSE
+#             }
+#             
+#             if (!is.null(interaction.depth)) s@interaction.depth <- interaction.depth
+#             else {
+#               if (!is.null(sobj)) {
+#                 if (!is.null(sobj@interaction.depth)) s@interaction.depth <- sobj@interaction.depth
+#               }
+#             }
+#             s
+#           }
+# )
+# #----------------
 if (!isGeneric("sdmSetting")) {
   setGeneric("sdmSetting", function(formula,data,methods,interaction.depth=1,n=1,replication=NULL,
                                     cv.folds=NULL,test.percent=NULL,bg=NULL,bg.n=NULL,var.importance=NULL,response.curve=TRUE,
-                                    var.selection=FALSE,ncore=1L,...)
+                                    var.selection=FALSE,ncore=1L,modelSettings=NULL,seed=NULL,...)
     standardGeneric("sdmSetting"))
 }
 
 setMethod('sdmSetting', signature(formula='ANY','sdmdata','character'), 
           function(formula,data,methods,interaction.depth=1,n=1,replication=NULL,
                    cv.folds=NULL,test.percent=NULL,bg=NULL,bg.n=NULL,var.importance=NULL,response.curve=TRUE,
-                   var.selection=FALSE,ncore=1L,...) {
+                   var.selection=FALSE,ncore=1L,modelSettings=NULL,seed=NULL,...) {
             
             if (!.sdmOptions$getOption('sdmLoaded')) .addMethods()
             
@@ -503,8 +784,8 @@ setMethod('sdmSetting', signature(formula='ANY','sdmdata','character'),
                 dot <- dot[-which(ndot == '')]
                 ndot <- names(dot)
               }
-              
-              a <- c('interaction.depth','replication','cv.folds','test.percent','bg','bg.n','var.importance','response.curve','var.selection','ncore')
+                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  
+              a <- c('interaction.depth','replication','cv.folds','test.percent','bg','bg.n','var.importance','response.curve','var.selection','ncore','modelSettings','seed')
               ndot <- .pmatch(ndot,a)
               w <- !is.na(ndot)
               if (length(w) > 0) {
@@ -533,6 +814,8 @@ setMethod('sdmSetting', signature(formula='ANY','sdmdata','character'),
                     else if (nd == 'var.importance' && is.null(var.importance)) var.importance <- dot[[nd]]
                     else if (nd == 'response.curve' && response.curve && is.logical(dot[[nd]])) response.curve <- dot[[nd]]
                     else if (nd == 'var.selection' && !var.selection && is.logical(dot[[nd]])) var.selection <- dot[[nd]]
+                    else if (nd == 'modelSettings' && is.null(modelSettings)) modelSettings <- dot[[nd]]
+                    else if (nd == 'seed' && is.null(seed)) seed <- dot[[nd]]
                   }
                 }
               }
@@ -542,10 +825,9 @@ setMethod('sdmSetting', signature(formula='ANY','sdmdata','character'),
             m <- .methodFix(methods)
             if (any(is.na(m))) stop(paste('methods',paste(methods[is.na(m)],collapse=', '),'do not exist!'))
             m <- unique(m)
-            
+            #---------
             s <- new('.sdmCorSetting',methods=m)
-            s@distribution <- .getSpeciesDistribution(data)
-            
+            #---------
             if (missing(formula)) {
               if (!is.null(sobj)) {
                 if (all(sobj@sdmFormula@vars %in% data@features.name)) s@sdmFormula <- sobj@sdmFormula
@@ -567,20 +849,19 @@ setMethod('sdmSetting', signature(formula='ANY','sdmdata','character'),
             }
             
             s@featuresFrame <- .getFeaturetype(data,s@sdmFormula)  
-              
+            #---------
+            s@distribution <- .getSpeciesDistribution(data,sp=s@sdmFormula@species)
+            #---------
             if (!is.null(test.percent)) s@test.percentage <- test.percent
             else {
               if (!is.null(sobj)) {
                 if (!is.null(sobj@test.percent)) s@test.percentage <- sobj@test.percent
               }
             }
-            
-            s@interaction.depth <- interaction.depth
-            if (interaction.depth ==1 && !is.null(sobj) && !is.null(sobj@interaction.depth)) s@interaction.depth <- sobj@interaction.depth
-            
+            #---------
             s@ncore <- ncore
             if (ncore == 1L && !is.null(sobj) && length(sobj@ncore) == 1) s@ncore <- sobj@ncore
-            
+            #---------
             if (!is.null(replication)) {
               nx <- .replicate.methodFix(replication)
               if (any(is.na(nx))) warning(paste(paste(replication[is.na(nx)],collapse=', '),'methods in replication are not found [They are ignored!]'))
@@ -611,7 +892,7 @@ setMethod('sdmSetting', signature(formula='ANY','sdmdata','character'),
             if (!is.null(s@cv.folds) && !"cross_validation" %in% s@replicate) {
               s@replicate <- c("cross_validation",s@replicate)
             }
-            
+            #---------
             if (!is.null(bg)) s@pseudo.absence.methods <- bg
             else {
               if (!is.null(sobj)) {
@@ -627,25 +908,31 @@ setMethod('sdmSetting', signature(formula='ANY','sdmdata','character'),
                 s@n.pseudo.absence <- 1000
               }
             }
+            #---------
             if (!is.null(var.importance)) s@varImportance.methods <- var.importance
             else {
               if (!is.null(sobj)) {
                 if (!is.null(sobj@varImportance.methods)) s@varImportance.methods <- sobj@varImportance.methods
               }
             }
+            #---------
             if (response.curve) s@response.curve <- TRUE
             else {
               if (!is.null(sobj)) {
                 if (!is.null(sobj@response.curve) && sobj@response.curve) s@response.curve <- sobj@response.curve
               } else s@response.curve <- FALSE
             }
-            
+            #---------
             if (var.selection) s@var.selection <- TRUE
             else {
               if (!is.null(sobj)) {
                 if (!is.null(sobj@var.selection) && sobj@var.selection) s@var.selection <- sobj@var.selection
               } else s@var.selection <- FALSE
             }
+            #---------
+            #s@interaction.depth <- interaction.depth
+            #if (interaction.depth ==1 && !is.null(sobj) && !is.null(sobj@interaction.depth)) s@interaction.depth <- sobj@interaction.depth
+            #---------
             
             if (!is.null(interaction.depth)) s@interaction.depth <- interaction.depth
             else {
@@ -653,6 +940,54 @@ setMethod('sdmSetting', signature(formula='ANY','sdmdata','character'),
                 if (!is.null(sobj@interaction.depth)) s@interaction.depth <- sobj@interaction.depth
               }
             }
+            #---------
+            if (!is.null(modelSettings) && inherits(modelSettings,'list')) {
+              .ms <- names(modelSettings)
+              if (!is.null(.ms)) {
+                .ms <- .methodFix(.ms)
+                if (!all(.ms %in% s@methods)) warning(paste('the models in the modelSettings:',paste0(names(modelSettings)[!.ms %in% s@methods],collapse = ', '),'are not selected in the methods, or do not exitst!'))
+                w <- which(.ms %in% s@methods)
+                if (length(w) > 0) {
+                  .ms <- .ms[w]
+                  modelSettings <- modelSettings[w]
+                  names(modelSettings) <- .ms
+                  ww <- c()
+                  for (i in seq_along(.ms)) {
+                    if(!inherits(modelSettings[[.ms[i]]],'list')) ww <- c(ww,i)
+                  }
+                  
+                  if (length(ww) > 0) {
+                    if (length(ww) < length(modelSettings)) {
+                      warning(paste('the modelSettings for the items:',paste(.ms[ww],collapse = ','),'are not a list, and so they are ignored!'))
+                      modelSettings <- modelSettings[-ww]
+                    } else {
+                      warning('the arguments for each method in the modelSettings should be introduced using a list; modelSettings is ignored!')
+                      modelSetting <- NULL
+                    }
+                  }
+                } else modelSetting <- NULL
+                
+              } else warning('modelSettings is not in the right structure, so it is ignored!')
+              
+              if (!is.null(modelSettings)) {
+                s@modelSettings <- modelSettings
+              }
+            } else {
+              if (!is.null(sobj)) {
+                if (!is.null(sobj@modelSettings)) s@modelSettings <- sobj@modelSettings
+              }
+            }
+            #---------
+            if (!is.null(seed)) {
+              if (is.logical(seed)) seed <- sample(100000,1)
+              else if (!is.numeric(seed)) seed <- NULL
+              s@seed <- seed
+            } else {
+              if (!is.null(sobj)) {
+                if (!is.null(sobj@seed)) s@seed <- sobj@seed
+              }
+            }
+            #-------------
             s
           }
 )
@@ -662,9 +997,9 @@ if (!isGeneric("sdm")) {
     standardGeneric("sdm"))
 }
 
-setMethod('sdm', signature(formula='formula',data='sdmdata',methods='character'), 
+setMethod('sdm', signature(formula='ANY',data='sdmdata',methods='character'), 
           function(formula,data,methods,...) {
-            a <- c('interaction.depth','n','replication','cv.folds','test.percent','bg','bg.n','var.importance','response.curve','var.selection','setting','ncore')
+            a <- c('interaction.depth','n','replication','cv.folds','test.percent','bg','bg.n','var.importance','response.curve','var.selection','setting','ncore','modelSettings','seed')
             .sdm...temp <- NULL; rm(.sdm...temp)
             dot <- list(...)
             ndot <- names(dot)
@@ -676,6 +1011,8 @@ setMethod('sdm', signature(formula='formula',data='sdmdata',methods='character')
               names(dot) <- ndot
             }
             
+            if (missing(formula)) formula <- NULL
+            
             dot$data <- data
             dot$formula <- formula
             dot$methods <- methods
